@@ -27,6 +27,9 @@ class TestLuaRuntime(unittest.TestCase):
     def test_eval(self):
         self.assertEqual(2, self.lua.eval('1+1'))
 
+    def test_eval_multi(self):
+        self.assertEqual((1,2,3), self.lua.eval('1,2,3'))
+
     def test_execute(self):
         self.assertEqual(2, self.lua.execute('return 1+1'))
 
@@ -123,10 +126,18 @@ class TestLuaRuntime(unittest.TestCase):
         self.assertEqual(2, eval('1+1'))
         self.assertEqual(2, self.lua.eval('python.eval("1+1")'))
 
-    def test_len_table(self):
+    def test_len_table_array(self):
         table = self.lua.eval('{1,2,3,4,5}')
         self.assertEqual(5, len(table))
 
+    def test_len_table_dict(self):
+        table = self.lua.eval('{a=1, b=2, c=3}')
+        self.assertEqual(0, len(table)) # as returned by Lua's "#" operator
+
+    def test_len_table(self):
+        table = self.lua.eval('{1,2,3,4, a=1, b=2, c=3}')
+        self.assertEqual(4, len(table)) # as returned by Lua's "#" operator
+
     def test_iter_table(self):
         table = self.lua.eval('{1,2,3,4,5}')
         self.assertEqual([1,2,3,4,5], list(table))
@@ -161,14 +172,14 @@ class TestLuaRuntime(unittest.TestCase):
         table = self.lua.eval('{%s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
         l = list(table.values())
         l.sort()
-        self.assertEqual(range(len('abcdefg')), l)
+        self.assertEqual(list(range(len(keys))), l)
 
     def test_iter_table_items(self):
         keys = list('abcdefg')
         table = self.lua.eval('{%s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
         l = list(table.items())
         l.sort()
-        self.assertEqual(zip(keys,range(len(keys))), l)
+        self.assertEqual(list(zip(keys,range(len(keys)))), l)
 
     def test_error_iter_function(self):
         func = self.lua.eval('function() return 1 end')
@@ -190,6 +201,34 @@ class TestLuaRuntime(unittest.TestCase):
         table = self.lua.eval('{}')
         self.assertEqual('<Lua table at ', str(table)[:14])
 
+    def test_create_table_args(self):
+        table = self.lua.table(1,2,3,4,5,6)
+        self.assertEqual(1, table[1])
+        self.assertEqual(3, table[3])
+        self.assertEqual(6, table[6])
+
+        self.assertEqual(6, len(table))
+
+    def test_create_table_kwargs(self):
+        table = self.lua.table(a=1, b=20, c=300)
+        self.assertEqual(  1, table['a'])
+        self.assertEqual( 20, table['b'])
+        self.assertEqual(300, table['c'])
+
+        self.assertEqual(0, len(table))
+
+    def test_create_table_args_kwargs(self):
+        table = self.lua.table(1,2,3,4,5,6, a=100, b=200, c=300)
+        self.assertEqual(1, table[1])
+        self.assertEqual(3, table[3])
+        self.assertEqual(6, table[6])
+
+        self.assertEqual(100, table['a'])
+        self.assertEqual(200, table['b'])
+        self.assertEqual(300, table['c'])
+
+        self.assertEqual(6, len(table))
+
     def test_getattr(self):
         stringlib = self.lua.eval('string')
         self.assertEqual('abc', stringlib.lower('ABC'))
@@ -212,6 +251,20 @@ class TestLuaRuntime(unittest.TestCase):
         self.assertEqual('light speed', table['const2']['name'])
         self.assertEqual(3e8, table['const2']['value'])
 
+    def test_getitem_array(self):
+        table = self.lua.eval('{1,2,3,4,5,6,7,8,9}')
+        self.assertEqual(1, table[1])
+        self.assertEqual(5, table[5])
+        self.assertEqual(9, len(table))
+
+    def test_setitem_array(self):
+        table = self.lua.eval('{1,2,3,4,5,6,7,8,9}')
+        self.assertEqual(1, table[1])
+        table[1] = 0
+        self.assertEqual(0, table[1])
+        self.assertEqual(2, table[2])
+        self.assertEqual(9, len(table))
+
     def test_setattr_table(self):
         table = self.lua.eval('{ const={ name="Pi", value=3.1415927 }, const2={ name="light speed", value=3e8 }, val=1 }')
 
@@ -247,6 +300,11 @@ class TestLuaRuntime(unittest.TestCase):
         lua_globals = self.lua.globals()
         self.assertNotEqual(None, lua_globals.unpack)
 
+    def test_globals_attrs_call(self):
+        lua_globals = self.lua.globals()
+        self.assertNotEqual(None, lua_globals.string)
+        self.assertEqual('test', lua_globals.string.lower("TEST"))
+
     def test_require(self):
         stringlib = self.lua.require('string')
         self.assertNotEqual(None, stringlib)
@@ -480,7 +538,7 @@ class TestThreading(unittest.TestCase):
         self.assertEqual(185925, results[0])
 
     def test_threading_iter(self):
-        values = range(1,100)
+        values = list(range(1,100))
         lua = lupa.LuaRuntime()
         table = lua.eval('{%s}' % ','.join(map(str, values)))
         self.assertEqual(values, list(table))
